from super_scad.boolean.Union import Union


class Compound(Union):
    """
    Creates a union of all its child widget. This is the sum of all children (logical or). See
    https://en.wikibooks.org/wiki/OpenSCAD_User_Manual/CSG_Modelling#union.
    """
    pass

# ----------------------------------------------------------------------------------------------------------------------
